from collections import namedtuple
from django.core.cache import cache
import requests
import time
from . import settings

Token = namedtuple('Token', ('token', 'referenceTimestamp', 'expiresIn'))


class TokenGenerator(object):
    client_id = settings.EXACT_TARGET_CLIENT_ID
    client_secret = settings.EXACT_TARGET_CLIENT_SECRET
    url = 'https://auth.exacttargetapis.com/v1/requestToken'

    def generate(self):
        t = self._request()
        return t.token

    def headers(self):
        headers = {
            'Authorization': 'Bearer %s' % self.generate()
        }
        return headers

    def _request(self):
        resp = requests.post(self.url, json={
            "clientId": self.client_id,
            "clientSecret": self.client_secret,
        })

        resp.raise_for_status()
        data = resp.json()
        token = Token(
            token=data.get("accessToken"),
            referenceTimestamp=time.time(),
            expiresIn=data.get("expiresIn"))
        return token


class CachedTokenGenerator(TokenGenerator):
    tolerance = 10  # Cache the token for 10 seconds less than it's valid

    def _gen_key(self):
        return 'exacttarget_%s' % self.client_id

    def _request(self):
        key = self._gen_key()
        token = cache.get(key)
        if token is None:
            token = super()._request()
            cache.set(key, token, token.expiresIn - self.tolerance)
        return token



from . import token, exceptions
import requests
import logging
import sentry_sdk



logger = logging.getLogger(__name__)



class TriggeredSend(object):
    url_base = 'https://www.exacttargetapis.com/messaging/v1/messageDefinitionSends'

    def __init__(self, external_key):
        self.external_key = external_key
        self.token = token.CachedTokenGenerator()


    def dispatch(self, email, attrs):
        url = '%s/key:%s/send' % (self.url_base, self.external_key)
        headers = self.token.headers()
        req_data = {
            "To": {
                "Address": email,
                "SubscriberKey": email,
                "ContactAttributes": {
                    "SubscriberAttributes": attrs
                },
            },
            "OPTIONS": {
                "RequestType": "SYNC"
            },
        }

        resp = requests.post(url, headers=headers, json=req_data)
        resp.raise_for_status()
        reply = resp.json()
        with sentry_sdk.configure_scope() as scope:
            scope.set_extra("reply_data", reply)
        for response in reply['responses']:
            if response['hasErrors']:
                error_messages = response.get('messageErrors', [])
                # set of error codes in messages
                error_codes = {err.get('messageErrorCode') for err in error_messages}
                # coerce ET_IGNORED_ERROR_CODES into set
                from . import settings
                codes_to_ignore = set(settings.ET_IGNORED_ERROR_CODES)

                # if response has any ignorable codes... set intersection
                if len(codes_to_ignore & error_codes) > 0:
                    # log error, but don't send ignored error codes to Sentry
                    logger.warn("Suppressed exception for ET API exception. Error response: {}".format(reply))
                else:
                    sentry_sdk.capture_message('Error occurred while submitting subscriber to exact target')
                    reg_messages = response.get('messages')
                    # sometimes there are structured errors
                    if len(error_messages):
                        raise exceptions.TriggeredSendException(error_messages[0]['messageErrorStatus'])
                    # sometimes there are plain text errors
                    elif len(reg_messages):
                        raise exceptions.TriggeredSendException(reg_messages[0])
                    else:
                        raise exceptions.TriggeredSendException('Unknown TriggeredSend Error Occurred')
        return reply



from setuptools import setup, Distribution

Distribution().fetch_build_eggs('versiontag')

from versiontag import get_version, cache_git_tag  # NOQA
import codecs  # NOQA
import os.path  # NOQA


packages = [
    'exacttarget',
]

install_requires = [
    'Django>=1.11',
    'sentry-sdk>=0.5.5',
    'requests>=2.9.1',
    'simplejson>=3.8.2',
]

extras_require = {
    'development': [
        'flake8>=3.2.1',
        'requests_mock>=0.7.0',
        'mock>=2.0.0',
    ],
}


def fpath(name):
    return os.path.join(os.path.dirname(__file__), name)


def read(fname):
    return codecs.open(fpath(fname), encoding='utf-8').read()


cache_git_tag()

setup(
    name='django-exact-target',
    description="Integration between django and the SalesForce ExactTarget REST API",
    version=get_version(pypi=True),
    long_description=open('README.rst').read(),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: ISC License (ISCL)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
    ],
    author='Craig Weber',
    author_email='crgwbr@gmail.com',
    url='https://gitlab.com/thelabnyc/django-exact-target',
    license='ISC',
    packages=packages,
    install_requires=install_requires,
    extras_require=extras_require
)
